//------------------------------------------------------------------------------
// File: WipeNum.cpp
//
// Desc: DirectShow sample code - Transition Previewer Sample
//
// Copyright (c) Microsoft Corporation.  All rights reserved.
//------------------------------------------------------------------------------

#include "Transviewer.h"
#include "PropSetter.h"


// This list contains informal names for the various SMPTE wipes,
// along with the corresponding wipe number. (Wipe numbers are not
// continuous, so we can't just index into an array of strings...)

WipeNumInfo g_SMPTEWipes[] =
{
    {1,     TEXT("1.   Horizontal") },
    {2,     TEXT("2.   Vertical") },
    {3,     TEXT("3.   Upper left") },
    {4,     TEXT("4.   Upper right") },
    {5,     TEXT("5.   Lower right") },
    {6,     TEXT("6.   Lower left") },
    {7,     TEXT("7.   Four corners") },
    {8,     TEXT("8.   Four squares") },
    {21,    TEXT("21.  Barn doors, vertical") },
    {22,    TEXT("22.  Barn doors, horizontal") },
    {23,    TEXT("23.  Top center") },
    {24,    TEXT("24.  Right center") },
    {25,    TEXT("25.  Bottom center") },
    {26,    TEXT("26.  Left center") },
    {41,    TEXT("41.  Diagonal, NW to SE") },
    {42,    TEXT("42.  Diagonal, NE to SW") },
    {43,    TEXT("43.  Triangles, top/bottom") },
    {44,    TEXT("44.  Triangles, left/right") },
    {45,    TEXT("45.  Diagonal stripe, SW to NE") },
    {46,    TEXT("46.  Diagonal stripe, NW to SE") },
    {47,    TEXT("47.  Cross") },
    {48,    TEXT("48.  Diamond Box") },
    {61,    TEXT("61.  Wedge, top") },
    {62,    TEXT("62.  Wedge, right") },
    {63,    TEXT("63.  Wedge, bottom") },
    {64,    TEXT("64.  Wedge, left") },
    {65,    TEXT("65.  V") },
    {66,    TEXT("66.  V, right") },
    {67,    TEXT("67.  V, inverted") },
    {68,    TEXT("68.  V, left") },
    {71,    TEXT("71.  Sawtooth, left") },
    {72,    TEXT("72.  Sawtooth, top") },
    {73,    TEXT("73.  Sawtooth, vertical") },
    {74,    TEXT("74.  Sawtooth, horizontal") },
    {101,   TEXT("101. Box") },
    {102,   TEXT("102. Diamond") },
    {103,   TEXT("103. Triangle, up") },
    {104,   TEXT("104. Triangle, right") },
    {105,   TEXT("105. Triangle, bottom") },
    {106,   TEXT("106. Triangle, left") },
    {107,   TEXT("107. Arrow head, up") },
    {108,   TEXT("108. Arrow head, right") },
    {109,   TEXT("109. Arrow head, down") },
    {110,   TEXT("110. Arrow head, left") },
    {111,   TEXT("111. Pentagon, up") },
    {112,   TEXT("112. Pentagon, down") },
    {113,   TEXT("113. Hexagon") },
    {114,   TEXT("114. Hexagon, rotated") },
    {119,   TEXT("119. Circle") },
    {120,   TEXT("120. Oval, horizontal") },
    {121,   TEXT("121. Oval, vertical") },
    {122,   TEXT("122. Eye, horizontal") },
    {123,   TEXT("123. Eye, vertical") },
    {124,   TEXT("124. Rounded rectangle, horizontal") },
    {125,   TEXT("125. Rounded rectangle, vertical") },
    {127,   TEXT("127. 4-point star") },
    {128,   TEXT("128. 4-point star") },
    {129,   TEXT("129. 6-point star") },
    {130,   TEXT("130. Heart") },
    {131,   TEXT("131. Keyhole") },
    {201,   TEXT("201. Radial, 12 o'clock") },
    {202,   TEXT("202. Radial, 3 o'clock") },
    {203,   TEXT("203. Radial, 6 o'clock") },
    {204,   TEXT("204. Radial, 9 o'clock") },
    {205,   TEXT("205. Radial, 12 + 6 o'clock") },
    {206,   TEXT("206. Radial, 3 + 9 o'clock") },
    {207,   TEXT("207. Radial, 4-way") },
    {211,   TEXT("211. Radial, left-right, top") },
    {212,   TEXT("212. Radial, up-down, right") },
    {213,   TEXT("213. Radial, left-right, top-bottom") },
    {214,   TEXT("214. Radial, up-down, left-right") },
    {221,   TEXT("221. Radial, top") },
    {222,   TEXT("222. Radial, right") },
    {223,   TEXT("223. Radial, bottom") },
    {224,   TEXT("224. Radial, left") },
    {225,   TEXT("225. Radial, top clockwise, bottom clockwise") },
    {226,   TEXT("226. Radial, left clockwise, right clockwise") },
    {227,   TEXT("227. Radial, top clockwise, bottom anticlockwise") },
    {228,   TEXT("228. Radial, left clockwise, right anticlockwise") },
    {231,   TEXT("231. Radial, top split") },
    {232,   TEXT("232. Radial, right split") },
    {233,   TEXT("233. Radial, bottom split") },
    {234,   TEXT("234. Radial, left split") },
    {235,   TEXT("235. Radial, top-bottom split") },
    {236,   TEXT("236. Radial, left-right split") },
    {241,   TEXT("241. Radial, top-left corner") },
    {242,   TEXT("242. Radial, bottom-left corner") },
    {243,   TEXT("243. Radial, bottom-right corner") },
    {244,   TEXT("244. Radial, top-right corner") },
    {245,   TEXT("245. Radial, top-left, bottom-right") },
    {246,   TEXT("246. Radial, bottom-left, top-right") },
    {251,   TEXT("251. Center radial, top") },
    {252,   TEXT("252. Center radial, left") },
    {253,   TEXT("253. Center radial, bottom") },
    {254,   TEXT("254. Center radial, right") },
    {261,   TEXT("261. Box radial, right") },
    {262,   TEXT("262. Box radial, top") },
    {263,   TEXT("263. Center radial, top, bottom") },
    {264,   TEXT("264. Center radial, left, right") },
    {301,   TEXT("301. Matrix, horizontal") },
    {302,   TEXT("302. Matrix, vertical") },
    {303,   TEXT("303. Matrix, diagonal, top-left") },
    {304,   TEXT("304. Matrix, diagonal, top-right") },
    {305,   TEXT("305. Matrix, diagonal, bottom-right") },
    {306,   TEXT("306. Matrix, diagonal, bottom-left") },
    {310,   TEXT("310. Matrix, clockwise top-left") },
    {311,   TEXT("311. Matrix, clockwise top-right") },
    {312,   TEXT("312. Matrix, clockwise bottom-right") },
    {313,   TEXT("313. Matrix, clockwise bottom-left") },
    {314,   TEXT("314. Matrix, anticlockwise top-left") },
    {315,   TEXT("315. Matrix, anticlockwise top-right") },
    {316,   TEXT("316. Matrix, anticlockwise bottom-right") },
    {317,   TEXT("317. Matrix, anticlockwise bottom-left") },
    {320,   TEXT("320. Matrix, vertical top-left, top-right") },
    {321,   TEXT("321. Matrix, vertical bottom-left, bottom-right") },
    {322,   TEXT("322. Matrix, vertical top-left, bottom-right") },
    {323,   TEXT("323. Matrix, vertical bottom-left, top-right") },
    {324,   TEXT("324. Matrix, horizontal top-left, bottom-left") },
    {325,   TEXT("325. Matrix, horizontal top-right, bottom-right") },
    {326,   TEXT("326. Matrix, horizontal top-left, bottom-right") },
    {327,   TEXT("327. Matrix, horizontal top-right, bottom-left") },
    {328,   TEXT("328. Matrix, diagonal bottom-left, top-right") },
    {329,   TEXT("329. Matrix, diagonal top-left, bottom-right") },
    {340,   TEXT("340. Matrix, top double spiral") },
    {341,   TEXT("341. Matrix, bottom double spiral") },
    {342,   TEXT("342. Matrix, left double spiral") },
    {343,   TEXT("343. Matrix, right double spiral") },
    {344,   TEXT("344. Matrix, quad spiral, top-bottom") },
    {345,   TEXT("345. Matrix, quad spiral, left-right") },
    {350,   TEXT("350. Waterfall, left") },
    {351,   TEXT("351. Waterfall, right") },
    {352,   TEXT("352. Waterfall, horizontal, left") },
    {353,   TEXT("353. Waterfall, horizontal, right") }
};


int g_iNumWipes =   132;




            